##-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
# The RNBIP Emulator
# This Multi threaded module 1: 1 emulates the operation of the RNBIP Micro - Controller
# Author::@CommandPaul(Paul George)# Junior Year 2016 - 17
# Shiv Nadar University 
##-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

##-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
#   This Python Script is hierarchially seperated into the original modulles of
#   the architecture, all original signals have been preserved the encoded data
#   can be visualised in VedantCJ(Vedant Chakravarty `s) Visualisation module
#   Developed in C++ using the QT Framework , The input to this file should be
#   in the format output by the Pn337(Prasanna Natarajan) Assembler .
##-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

#---- Fixit Comments Remove from final
# Re-Write this in cpp
#---- End of Fixit Comments

#   Note :: The killthr flag is negative logic

#   Import Libraries
import time
from multiprocessing import Process,Value,Queue,Array,Barrier
from ctypes import c_ubyte,c_char,c_int
import ctypes
import multiprocessing, logging
import random
import pdb
import os
import datetime
import platform
import re
import sys
# import importlib

from rnbip_emulator.rnbip_isa import Architectural_state,Microarchitectural_state
# if platform.system == "Linux":
#   from emulator_io import IO_Interface
# Block Design
# process declarition with all shared variables
# Procces tasks on current mmachine state
#process essentially creates a copy of all write affect control bits for that cycle # Write schedule protect
# Process state is recorded :D # essentially as an outcome of the previous state
# process Wait at Barrier for Synchronous
# Process executes Synchronous functions
# All mem writes are performed prep for next set of control siglanls is done
# wait for Kill Signal
# barrier 2

'''def Proc(StateVariableA,StateVariableB,Kill,Barrier1,Barrier2):
    
    # The design is assumed to be completly synchronous to an implict clock
    # Barriers are used as an analogue to Standard HDL Event Regions, 
    # which help us split the design simulation into independent phases 
    # helping us avoid race conditions on our lock free state variables

    < Initialization Phase >
    StateVariableA.set(0)
    StateVariableB.set(0)

    Barrier2.wait()
    a = StateVariableA.get()+1
    
    while(1):
      StateVariableB.put(a)
      Barrier1.wait()
    
      a = StateVariableA.get()+1
      StateVariableB.put(a)
      Barrier2.wait()
      if(Kill.value == 1 ):
        return
  '''

class single_bus_emulator:

  ## RNBIP Architectural State 
  ProgramCounter = Value(c_ubyte, 0, lock=False)
  StackPointer = Value(c_ubyte, 0, lock=False)
  RegisterArray = Array(c_ubyte,[1,2,3,4,5,6,7,8] , lock=False)
  FlagRegister = Array(c_ubyte,[1,0,0,1,1,0,0,1] , lock=False)
  Memory = Array(c_ubyte, 256, lock=False)
  IO_In = Array(c_ubyte,[0,0,0,0,0,0,0,0] , lock=False)
  IO_Out = Array(c_ubyte,[0,0,0,0,0,0,0,0] , lock=False)

  ## RNBIP MicroArchitectural State
  DataBus = Queue()
  IR_CCG = Queue()
  #SelectedFlag = Queue()
  AluOut = Value(c_ubyte, 0, lock=False) # make this a queue and put values spl case of no SAL for ClC case
  #AluOut = Value(c_ubyte, 0, lock=False)
  AluOpcode = Value(c_ubyte, 0, lock=False)
  CarryIn = Value(c_ubyte, 0, lock=False)
  InstructionRegister = Value(c_ubyte, 0, lock=False)
  OperandRegister = Value(c_ubyte, 0, lock=False)
  State = Value(c_ubyte, 0, lock=False)
  ControlBits = Array(c_char,b'1010010000100000000000', lock=False ) # needs to be Cached

  ## Model specific Variables
  Ncyc = 20
  Kill = Value(c_ubyte, 0, lock=False)
  #RegSel = Value(c_ubyte, 0, lock=False)#needs Cacheing
  
  ## Multi Processing and Synchronization State
  ThrCCG = Process()
  ThrIR = Process()
  ThrPC = Process()
  ThrMEM = Process()
  ThrRegArr = Process()
  ThrALU = Process()
  ThrSPtr = Process()
  ThrOpReg = Process()
  ThrIO = Process()
  Barrier1 = Barrier(10) # dont forget ot update :P
  Barrier2 = Barrier(10)

  ## Trace Vector
  TraceVector = []

  ## Housekeeping static members
  @staticmethod
  def parityOf(number):
    parity = 0
    int_type = number
    while (int_type > 0):
      parity = ~parity
      int_type = int_type & (int_type - 1)
    return(parity + 1)

  @staticmethod
  def file_len(fname):
  	i=0;
  	with fname as f:
  		for i, l in enumerate(f):
  			pass
  	return i + 1

  ## State machine static member functions

  @staticmethod
  def RegisterArrayModule(ControlBits,DataBus,RegisterArr,InstructionRegister,AluOut,Kill,Barrier1,Barrier2):
    #initialisation read from file
    RegisterArray = [0,1,2,3,4,5,6,7]
    for k in range(0,8):
      RegisterArray[k] = random.randint(0,255)
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      LR0 = ControlBits[18] #cached for Synchronous Protection
      LRN = ControlBits[20] #cached for Synchronous Protection
      SAL = ControlBits[17] #cached for Synchronous Protection
      REGSEL = InstructionRegister.value %8
      #print(LR0,LRN,SAL,REGSEL)

      if(ControlBits[19] == b'1'):
        DataBus.put(RegisterArray[0])

      if(ControlBits[21] == b'1'):
        DataBus.put(RegisterArray[REGSEL])

      Barrier1.wait()
      # Synchronous Here after
      if( LR0 == b'1' and LRN == b'1'):
        RegisterArray = [0,0,0,0,0,0,0,0]
      #a = AluOut.get() # a useless pop to clear the queue
      elif( LR0 == b'1' and SAL == b'0'):
        RegisterArray[0] = DataBus.get() # .value requirement ? Not Required :)
      elif( LRN == b'1' and SAL == b'0'):
        RegisterArray[REGSEL] = DataBus.get()  # .value requirement ? Not Required :)
      elif( LR0 == b'1' and SAL == b'1'):
        RegisterArray[0] = AluOut.value  # .value requirement ? Not Required :)
      elif( LRN == b'1' and SAL == b'1'):
        RegisterArray[REGSEL] = AluOut.value  # .value requirement ? Not Required :)
      for k in range(0,8):
        RegisterArr[k] = RegisterArray[k]
      # print(RegisterArray)

      Barrier2.wait()
      if(Kill.value == 1 ):
        #           print("Bye - Reg Arr")
        return
    return

  ## Additions for Prof. Biswas`s Requested Realtime IO Request Feature

  
    #IO_Interface_Update(IO_In,IO_Out,master)
  ## End of Additions - CommandPaul
  @staticmethod
  def IOModule(ControlBits,InstructionRegister,IO_In,IO_Out,DataBus,Kill,Barrier1,Barrier2):
    #initialisation
    IO_In  = [1,3,5,7,9,11,13,15]
    IO_Out = [2,4,6,8,10,12,14,16]
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      LOP = ControlBits[14] #cached for Synchronous Protection
      REGSEL = InstructionRegister.value%8 #cached for Synchronous Protection
      if(ControlBits[13] == b'1'):
        # Tkinter Code Here
        # interface = IO_Interface(IO_In,IO_Out)
        # interface.mainloop()
        # print(IO_In)
        DataBus.put(IO_In[REGSEL]) ## IOs are Effectively Looped Back
      Barrier1.wait()
      # Synchronous Here after
      if( LOP == b'1'):
        IO_Out[REGSEL] = DataBus.get() # .value requirement ? Not Required :)
        #interface = IO_Interface(IO_In,IO_Out)
        #interface.mainloop()
        #print(IO_Out)
      Barrier2.wait()
      if(Kill.value == 1 ): return
    return

  @staticmethod
  def OperandRegisterModule(ControlBits,OperandRegister,DataBus,Kill,Barrier1,Barrier2):
    #initialisation
    OperandRegister.value = 0
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      LOR = ControlBits[16] #cached for Synchronous Protection
      if(ControlBits[15] == b'1'):
        DataBus.put(OperandRegister.value)
      Barrier1.wait()
      # Synchronous Here after
      if( LOR == b'1'):
        OperandRegister.value = DataBus.get() # .value requirement ? Not Required :)
      Barrier2.wait()
      if(Kill.value == 1 ): return
    return

  @staticmethod
  def ALU_Module(ControlBits,DataBus,OperandRegister,InstructionRegister,CarryIn,AluOut,FlagRegister,Kill,Barrier1,Barrier2):
    #initialisation
    OperandRegister.value = 0
    ALU_imOut = c_int(0)
    FR = [1,0,0,1,1,0,0,1]
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      SAL = ControlBits[17] #cached for Synchronous Protection
      OpCode = InstructionRegister.value >> 4 #for faster Switch below :|
      #print(OpCode)
      if(SAL == b'1'):
        if   (OpCode == 0    ):  ALU_imOut = 0
        else :
          dataBus = DataBus.get()
          OperandReg = OperandRegister.value
          CarryIn = FR[2]
          if   (OpCode == 1    ):  ALU_imOut = dataBus
          elif (OpCode == 2    ):  ALU_imOut = (~dataBus & 0x1FF) # the 1FF is maintainance of a design descision
          elif (OpCode == 3    ):  ALU_imOut = OperandReg
          elif (OpCode == 4    ):  ALU_imOut = dataBus+1
          elif (OpCode == 5    ):  ALU_imOut = dataBus-1
          elif (OpCode == 6    ):  ALU_imOut = ((dataBus<<1)+CarryIn)
          elif (OpCode == 7    ):  ALU_imOut = (((dataBus%2)<<8)+(CarryIn<<7)+(dataBus>>1))
          elif (OpCode == 8    ):  ALU_imOut = dataBus+OperandReg
          elif (OpCode == 9    ):  ALU_imOut = OperandReg-dataBus
          elif (OpCode == 10   ):  ALU_imOut = dataBus+OperandReg+CarryIn
          elif (OpCode == 11   ):  ALU_imOut = OperandReg-dataBus-CarryIn
          elif (OpCode == 12   ):  ALU_imOut = dataBus&OperandReg
          elif (OpCode == 13   ):  ALU_imOut = dataBus|OperandReg
          elif (OpCode == 14   ):  ALU_imOut = dataBus^OperandReg
          elif (OpCode == 15   ):  ALU_imOut = ~(dataBus^OperandReg) & 0x1FF # the 1FF is maintainance of a design descision
        AluOut.value = ALU_imOut%256
        ALU_imOut = ALU_imOut%512

        #print(AluOut.value)
        #Flag Register Update #consoder making the flag pipe a queue
        #ALU_imOut = int(ALU_imOut)
        FR = [(ALU_imOut%256 == 0),(ALU_imOut%256 != 0),(ALU_imOut >= 256),not (ALU_imOut >= 256),((((ALU_imOut%256)&(1<<7))>>7)==0),(((~(ALU_imOut%256)&(1<<7))>>7)==0),(single_bus_emulator.parityOf(ALU_imOut%256)==0),(single_bus_emulator.parityOf(ALU_imOut%256)==1)]
        for k in range(0,8):
          FlagRegister[k] = FR[k]
        #print("FLags")
        #print(FlagRegister[4],end = " ")
        #print(FlagRegister[5])
        #print("ALU OUT  == = == ",end = " ")
        #print(FR,end = " ")
        #print(ALU_imOut)
        #print([chr(ALU_imOut / 256), ~chr(ALU_imOut / 256), (ALU_imOut == 0), ~(ALU_imOut == 0),~(ALU_imOut & (1 << 7)), (~ALU_imOut & (1 << 7)), (parityof(ALU_imOut)),~(parityof(ALU_imOut))])
      Barrier1.wait()
      # Synchronous Here after
      # no regs and synchronous activity in the ALU module ?
      Barrier2.wait()
      if(Kill.value == 1 ): return
    return
  
  @staticmethod
  def StackPointerModule(ControlBits,DataBus,StackPointer,ProgramCounter,Kill,Barrier1,Barrier2):
    #initialisation read from file
    StackPointer.value = 0
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      DSP = ControlBits[8] #cached for Synchronous Protection
      ISP = ControlBits[9] #cached for Synchronous Protection
      # i deleted ESP when removing EPC
      if(ControlBits[7] == b'1'):
        DataBus.put(StackPointer.value)
      Barrier1.wait()
      # Synchronous Here after
      if( DSP == b'1' and ISP == b'0'):
        StackPointer.value = (StackPointer.value - 1)%256 # .value requirement ? Not Required :)
      if( ISP == b'1' and DSP == b'0'):
        StackPointer.value = (StackPointer.value + 1)%256  # .value requirement ? Not Required :)
      if( ISP == b'1' and DSP == b'1'):
        StackPointer.value = DataBus.get()
      Barrier2.wait()
      if(Kill.value == 1 ): return
    return

  @staticmethod
  def MemoryModule(ControlBits,ProgramCounter,RegisterArray,StackPointer,DataBus,Memory,Kill,Barrier1,Barrier2):
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      DSP = ControlBits[8]
      WR = ControlBits[1] #cached for Synchronous Protection
      if(ControlBits[2]== b'1'): address = ProgramCounter.value
      elif(ControlBits[3]== b'1'): address =  StackPointer.value if (DSP == b'0') else (StackPointer.value - 1)%256
      else: address = RegisterArray[0]
      if(ControlBits[0] == b'1'):
        DataBus.put(Memory[address])
      Barrier1.wait()
      # Synchronous Here after
      if( WR == b'1'):
        Memory[address] = DataBus.get() #value requirement ? Not Required :)
      Barrier2.wait()
      if(Kill.value == 1 ):
        # while(counter<256): # Persist Memory state at the end of the smulation This needs the correct operation of the step function
        #   outFile = open("memory.bin","w")
        #   output = bin(Memory[counter])[2:].zfill(8)
        #   outFile.write(output+"\n");
        #   counter = counter+1
        return
    return

  @staticmethod
  def ProgramCounterModule(ControlBits,ProgramCounter,OperandRegister,DataBus,Kill,Barrier1,Barrier2):
    #initialisation read from file
    ProgramCounter.value = 0
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      LPC = ControlBits[6] #cached for Synchronous Protection
      IPC = ControlBits[5] #cached for Synchronous Protection
      if(ControlBits[4] == b'1'):
        DataBus.put(ProgramCounter.value)
      Barrier1.wait()
      # Synchronous Here after
      if (  LPC == b'1' and IPC == b'0'):
        #pdb.set_trace()
        #print(DataBus.get())
        ProgramCounter.value = DataBus.get() # .value requirement ? Not Required :)
      #print(ProgramCounter.value)
      elif( LPC == b'0' and IPC == b'1'):
        ProgramCounter.value = (ProgramCounter.value + 1)%256  # .value requirement ? Not Required :)
      elif( LPC == b'1' and IPC == b'1'):
        ProgramCounter.value = OperandRegister.value  # .value requirement ? Not Required :)
      Barrier2.wait()
      #print("ProgramCounter ",end=" ")
      #print(ProgramCounter.value)
      if(Kill.value == 1 ):
        # print("Bye - pc")
        return
    return

  @staticmethod
  def InstructionRegister_Module(ControlBits,InstructionRegister,DataBus,IR_CCG,Kill,Barrier1,Barrier2):
    InstructionRegister.value = 0
    counter = 0 # Diagnostic For runtime counting
    Barrier2.wait()
    while(1):
      L_IR = ControlBits[10] #cached for Synchronous Protection
      Barrier1.wait()
      if(L_IR == b'1' ) :
        InstructionRegister.value = DataBus.get()
        IR_CCG.put(InstructionRegister.value)
      Barrier2.wait()
      counter = counter +1
      if(Kill.value == 1 ):
        #print("bye - ir")
        #print(counter)
        return
    return

  @staticmethod
  def ControlCodeGenerator(ControlBits,IR_CCG,FlagRegister,State,Kill,Barrier1,Barrier2):
    #initialisation read from file

    counter=0
    IR = 0
    CCG_LUT = []
    FL = False
    # importlib.resources.path
    path = os.path.dirname(os.path.abspath(__file__))
    CCGConfFile = open(os.path.join(path,'machCode.bin'))
    while(counter<512):
      line = CCGConfFile.readline()
      Temp_Pointer = ctypes.create_string_buffer(line.encode())
      CCG_LUT.append(Temp_Pointer) #(int(line,2)) # control bit arr append and read subsequent # Computationally less expensive eats memory :P
      counter = counter+1


    #looks good :)
    State.value = 0
    ctypes.memmove(ControlBits,ctypes.create_string_buffer(b'1010010000100000000000'), 23) # Essentially Initialise for fetch
    # print("CCG Init Complete")
    Barrier2.wait()
    #Setup Main Loop
    while(1):
      #print("0  , 1 , 2  , 3  , 4  , 5  , 6  , 7  , 8  , 9  , 10 , 11 , 12 , 13 , 14 , 15 , 16 , 17 , 18 , 19 , 20 , 21 ")
      #print("RD , WR ,S_PC,S_SP,E_PC,I_PC,L_PC,E_SP,D_SP,I_SP,L_IR,E_FL,S_IF,E_IP,L_OP,E_OR,L_OR,S_AL,L_R0,E_R0,L_RN,E_RN")
      #for i in range(0,22):
      #   print(ControlBits[i],end=" ")
      #print(".")
      # cache existing control bbits and start acting on the same
      L_IR = ControlBits[10] #cached for synchronousnous Protection
      E_FL = ControlBits[11] #cached for Synchronous Protection
      S_IF = ControlBits[12] #cached for Synchronous Protection
      #Selected Flag :P
      #Fetch Logic (FL + E FL ) * S IF = 1. Selected FLAg  ?
      #print(S_IF)
      if (E_FL == b'1')  : FL =  FlagRegister[IR%8]

      FETCH = ((S_IF == b'1') and ((not (E_FL == b'1') or not(FL == True ))))
      Barrier1.wait()
      # if fetch condition detected then wait on the queue to read form IR   Reset State and push th bits for fetch
      # else issue
      if( FETCH == 1 ):
        #print("Fetch")
        State.value = 0
        ctypes.memmove(ControlBits,ctypes.create_string_buffer(b'1010010000100000000000'), 23)# eqv of checking the CCG LUT for Space 0 :P
      else:
        #print("No_Fetch")
        if (L_IR == b'1'):
          IR = IR_CCG.get()
        ctypes.memmove(ControlBits,CCG_LUT[IR*2 +State.value], 23)
        State.value = (State.value + 1) %2

      Barrier2.wait()
      if(Kill.value == 1 ):
        # print("Bye - ccg")
        return
    return
  #opens control Code generator csv aand line by line reads the machine states
  #it then moves through the state machien and issues control codes on the
  #  0, 1 ,2  , 3  , 4  , 5  , 6  , 7  , 8  , 9  , 10 , 11 , 12 , 13 , 14 , 15 , 16 , 17 , 18 , 19 , 20 , 21
  # RD,WR,S_PC,S_SP,E_PC,I_PC,L_PC,E_SP,D_SP,I_SP,L_IR,E_FL,S_IF,E_IP,L_OP,E_OR,L_OR,S+_AL,L_R0,E_R0,L_RN,E_RN

  def __init__(self):
    counter=0
    while(counter < 256):
      self.Memory[counter] = (int("00000000",2))
      counter = counter+1
    # print("emulator instantiated")

  def load_memory(self,memory_bin):
    counter=0
    self.memory_bin = memory_bin
    for line in self.memory_bin.splitlines():
      self.Memory[counter] = (int(line,2))
      counter = counter+1
    while(counter < 256):
      self.Memory[counter] = (int("00000000",2))
      counter = counter+1

  def start(self):
    #Thread Instantiation
    self.ThrCCG = Process(target=single_bus_emulator.ControlCodeGenerator,args=(self.ControlBits,self.IR_CCG,self.FlagRegister,self.State,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrIR = Process(target=single_bus_emulator.InstructionRegister_Module,args=(self.ControlBits,self.InstructionRegister,self.DataBus,self.IR_CCG,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrPC = Process(target=single_bus_emulator.ProgramCounterModule,args=(self.ControlBits,self.ProgramCounter,self.OperandRegister,self.DataBus,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrMEM = Process(target=single_bus_emulator.MemoryModule,args=(self.ControlBits,self.ProgramCounter,self.RegisterArray,self.StackPointer,self.DataBus,self.Memory,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrRegArr = Process(target=single_bus_emulator.RegisterArrayModule,args=(self.ControlBits,self.DataBus,self.RegisterArray,self.InstructionRegister,self.AluOut,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrALU = Process(target=single_bus_emulator.ALU_Module,args=(self.ControlBits,self.DataBus,self.OperandRegister,self.InstructionRegister,self.CarryIn,self.AluOut,self.FlagRegister,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrSPtr = Process(target=single_bus_emulator.StackPointerModule,args =(self.ControlBits,self.DataBus,self.StackPointer,self.ProgramCounter,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrOpReg = Process(target=single_bus_emulator.OperandRegisterModule,args=(self.ControlBits,self.OperandRegister,self.DataBus,self.Kill,self.Barrier1,self.Barrier2))
    self.ThrIO = Process(target=single_bus_emulator.IOModule,args=(self.ControlBits,self.InstructionRegister,self.IO_In,self.IO_Out,self.DataBus,self.Kill,self.Barrier1,self.Barrier2)) ## EDIT Regsel was not wired to the I0 Module

    #Reset Global State
    self.TraceVector.clear()
    self.Kill.value = 0
    self.ProgramCounter.value = 0
    self.StackPointer.value = 0
    for k in range(0,8):
      self.RegisterArray[k] = random.randint(0,255)
      self.IO_In[k] = 0
      self.IO_Out[k] = 0

    self.FlagRegister[0] = 1
    self.FlagRegister[1] = 0
    self.FlagRegister[2] = 0
    self.FlagRegister[3] = 1
    self.FlagRegister[4] = 1
    self.FlagRegister[5] = 0
    self.FlagRegister[6] = 0
    self.FlagRegister[7] = 1

    ## RNBIP MicroArchitectural State
    self.DataBus = Queue()
    self.IR_CCG = Queue()
    #SelectedFlag = Queue()
    self.AluOut.value = 0
    self.AluOpcode.value = 0
    self.CarryIn.value = 0
    self.InstructionRegister.value = 0
    self.OperandRegister.value = 0
    self.State.value = 0
    ctypes.memmove(self.ControlBits,ctypes.create_string_buffer(b'1010010000100000000000'), 23)


    #Thread Start
    self.ThrCCG.start()
    self.ThrIR.start()
    self.ThrPC.start()
    self.ThrMEM.start()
    self.ThrRegArr.start()
    self.ThrALU.start()
    self.ThrSPtr.start()
    self.ThrOpReg.start()
    self.ThrIO.start()

  def trace_vector_append(self):
    #Arch State
    words = []
    flags = []
    regs  = []
    IO_OUTPUT = []
    IO_INPUT =[]
    for word in self.Memory:
      words.append(word)
    for flag in self.FlagRegister: #ctype unpacking
      flags.append(flag == 1)
    for reg in self.RegisterArray: #ctype unpacking
      regs.append(reg)
    for io_inB in self.IO_In: #ctype unpacking
      IO_INPUT.append(io_inB)
    for io_outB in self.IO_Out: #ctype unpacking
      IO_OUTPUT.append(io_outB)

    arch = Architectural_state(
      PC=self.ProgramCounter.value,
      SP=self.StackPointer.value,
      Registers=regs.copy(),
      Flags=flags.copy(),
      Memory=words.copy(),
      IO_In=IO_INPUT.copy(),
      IO_Out=IO_OUTPUT.copy())
    
    del words, flags, regs , IO_INPUT , IO_OUTPUT
    
    #March State
    control_bits = []
    for cbit in self.ControlBits: #ctype unpacking
      control_bits.append(cbit == b'1')
    
    march = Microarchitectural_state(
      AluOut=self.AluOut.value,
      AluOpcode=self.AluOpcode.value,
      CarryIn=self.CarryIn.value,
      InstructionRegister=self.InstructionRegister.value,
      OperandRegister=self.OperandRegister.value,
      State=self.State.value,
      ControlBits=control_bits.copy())

    del control_bits

    self.TraceVector.append((arch,march))

  def step_n(self,Ncyc):
    N = Ncyc
    while(N>0):
      self.Barrier2.wait()
      self.trace_vector_append()
      self.Barrier1.wait()
      N = N-1
    # self.Kill.value  = 1
    # print(self.Barrier2.n_waiting)
    # self.Barrier2.wait()
    #Barrier2.wait()
    #print("End Cycle")

  def end_emulation(self):
    self.Kill.value  = 1
    self.Barrier2.wait()
    #Join threads and Prepare for Exit
    self.ThrCCG.join()
    self.ThrIR.join()
    self.ThrPC.join()
    self.ThrMEM.join()
    self.ThrRegArr.join()
    self.ThrALU.join()
    self.ThrSPtr.join()
    self.ThrOpReg.join()
    self.ThrIO.join()
    
    # self.DataBus.close()
    while(not self.DataBus.empty()):
      self.DataBus.get()
    # self.DataBus.join_thread()

    # self.IR_CCG.close()
    while(not self.IR_CCG.empty()):
      self.IR_CCG.get()
    # self.IR_CCG.join_thread()

    #print("No issue in init :D")
    # print (self.TraceVector)



import sys
import os
import time
from rnbip_emulator.rnbip_isa import prettyPrintOpDecode

from PyQt5.QtWidgets import (QWidget, QToolTip,
    QPushButton, QApplication, QMainWindow, QTableWidgetItem,
    QDesktopWidget)
from PyQt5.QtGui import (QFont, QPixmap, QPalette)
from PyQt5.QtCore import (Qt, QRect,QPoint,QSize)

from rnbip_emulator import rnbip_mainwindow_ui

from pkg_resources import resource_filename

QApplication.setAttribute(Qt.AA_EnableHighDpiScaling, True) #enable highdpi scaling
QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps, True) 

### Global State
path = os.path.dirname(os.path.abspath(__file__))

### QT GUI MainWinDow
class MainWidow(QMainWindow):

  def __init__(self,inst_emulator,inst_assembler,*args, **kwargs):
    super().__init__(*args, **kwargs)
    self.constructor()
    
    # Emulator Interface Setup
    # Connect Buttons
    self.ui.nextStage.clicked.connect(self.on_nextStage_clicked)
    self.ui.pushButton.clicked.connect(self.on_pushButton_clicked)
    self.ui.pushButton_2.clicked.connect(self.on_pushButton_2_clicked)
    self.ui.reset.clicked.connect(self.on_reset_clicked)
    self.ui.run10ClockCycle.clicked.connect(self.on_run10ClockCycle_clicked)

    self.inst_emulator = inst_emulator
    self.inst_emulator.start()
    self.inst_emulator.step_n(1)
    self.trace_index = 0
    self.last_compiled_mem = "00000000"
    self.log_string = "Welcome to the RNBIP Emulator \nEnter an Assembly program above and Hit the Compile button to get started !"
    self.ui.logBox.setText(self.log_string)
    self.inst_assembler = inst_assembler

  def constructor(self):
    self.ui = rnbip_mainwindow_ui.Ui_MainWidow()
    self.ui.setupUi(self)
    
    self.setWindowTitle("Single-Bus Processor")
    self.setWindowFlags(Qt.Window | Qt.WindowMinimizeButtonHint | Qt.WindowCloseButtonHint)
    
    self.show()
    
    h = self.height()-100
    w = self.width()-500
    pix = QPixmap(os.path.join(path,"asset/back_final.png"))
    self.ui.imageLabel_Source.setPixmap(pix.scaled(w,h,Qt.KeepAspectRatio))

    pix_1 = QPixmap(os.path.join(path,"asset/RD.png"))
    self.ui.imageLabel.setPixmap(pix_1.scaled(w,h,Qt.KeepAspectRatio))

    pix_2 = QPixmap(os.path.join(path,"asset/WR.png"))
    self.ui.imageLabel_2.setPixmap(pix_2.scaled(w,h,Qt.KeepAspectRatio))

    pix_3 = QPixmap(os.path.join(path,"asset/S_PC.png"))
    self.ui.imageLabel_3.setPixmap(pix_3.scaled(w,h,Qt.KeepAspectRatio))

    pix_4 = QPixmap(os.path.join(path,"asset/S_SP.png"))
    self.ui.imageLabel_4.setPixmap(pix_4.scaled(w,h,Qt.KeepAspectRatio))

    pix_5 = QPixmap(os.path.join(path,"asset/E_PC.png"))
    self.ui.imageLabel_5.setPixmap(pix_5.scaled(w,h,Qt.KeepAspectRatio))

    pix_6 = QPixmap(os.path.join(path,"asset/I_PC.png"))
    self.ui.imageLabel_6.setPixmap(pix_6.scaled(w,h,Qt.KeepAspectRatio))

    pix_7 = QPixmap(os.path.join(path,"asset/L_PC.png"))
    self.ui.imageLabel_7.setPixmap(pix_7.scaled(w,h,Qt.KeepAspectRatio))

    pix_8 = QPixmap(os.path.join(path,"asset/E_SP.png"))
    self.ui.imageLabel_8.setPixmap(pix_8.scaled(w,h,Qt.KeepAspectRatio))

    pix_9 = QPixmap(os.path.join(path,"asset/D_SP.png"))
    self.ui.imageLabel_9.setPixmap(pix_9.scaled(w,h,Qt.KeepAspectRatio))

    pix_10 = QPixmap(os.path.join(path,"asset/I_SP.png"))
    self.ui.imageLabel_10.setPixmap(pix_10.scaled(w,h,Qt.KeepAspectRatio))

    pix_11 = QPixmap(os.path.join(path,"asset/L_IR.png"))
    self.ui.imageLabel_11.setPixmap(pix_11.scaled(w,h,Qt.KeepAspectRatio))

    pix_12 = QPixmap(os.path.join(path,"asset/E_FL.png"))
    self.ui.imageLabel_12.setPixmap(pix_12.scaled(w,h,Qt.KeepAspectRatio))

    pix_13 = QPixmap(os.path.join(path,"asset/S_IF.png"))
    self.ui.imageLabel_13.setPixmap(pix_13.scaled(w,h,Qt.KeepAspectRatio))

    pix_14 = QPixmap(os.path.join(path,"asset/E_IP.png"))
    self.ui.imageLabel_14.setPixmap(pix_14.scaled(w,h,Qt.KeepAspectRatio))

    pix_15 = QPixmap(os.path.join(path,"asset/L_OP.png"))
    self.ui.imageLabel_15.setPixmap(pix_15.scaled(w,h,Qt.KeepAspectRatio))

    pix_16 = QPixmap(os.path.join(path,"asset/E_OR.png"))
    self.ui.imageLabel_16.setPixmap(pix_16.scaled(w,h,Qt.KeepAspectRatio))

    pix_17 = QPixmap(os.path.join(path,"asset/L_OR.png"))
    self.ui.imageLabel_17.setPixmap(pix_17.scaled(w,h,Qt.KeepAspectRatio))

    pix_18 = QPixmap(os.path.join(path,"asset/S_AL.png"))
    self.ui.imageLabel_18.setPixmap(pix_18.scaled(w,h,Qt.KeepAspectRatio))

    pix_19 = QPixmap(os.path.join(path,"asset/L_R0.png"))
    self.ui.imageLabel_19.setPixmap(pix_19.scaled(w,h,Qt.KeepAspectRatio))

    pix_20 = QPixmap(os.path.join(path,"asset/E_R0.png"))
    self.ui.imageLabel_20.setPixmap(pix_20.scaled(w,h,Qt.KeepAspectRatio))

    pix_21 = QPixmap(os.path.join(path,"asset/L_RN.png"))
    self.ui.imageLabel_21.setPixmap(pix_21.scaled(w,h,Qt.KeepAspectRatio))

    pix_22 = QPixmap(os.path.join(path,"asset/E_RN.png"))
    self.ui.imageLabel_22.setPixmap(pix_22.scaled(w,h,Qt.KeepAspectRatio))

    pix_23 = QPixmap(os.path.join(path,"asset/Databus.png"))
    self.ui.imageLabel_23.setPixmap(pix_23.scaled(w,h,Qt.KeepAspectRatio))

    pix_24 = QPixmap(os.path.join(path,"asset/L_SP.png"))
    self.ui.imageLabel_24.setPixmap(pix_24.scaled(w,h,Qt.KeepAspectRatio))

    # killLRN =  QPixmap(os.path.join(path,"imgasset//killLRN.png"))
    # self.ui.imageLabel_25.setPixmap(killLRN.scaled(w,h,Qt.KeepAspectRatio))

    pix_26 = QPixmap(os.path.join(path,"asset/killLRN2.png"))
    self.ui.imageLabel_26.setPixmap(pix_26.scaled(w,h,Qt.KeepAspectRatio))

    pix_27 = QPixmap(os.path.join(path,"asset/killLR0.png"))
    self.ui.imageLabel_27.setPixmap(pix_27.scaled(w,h,Qt.KeepAspectRatio))

    pix_28 = QPixmap(os.path.join(path,"asset/C_yes.png"))
    self.ui.imageLabel_28.setPixmap(pix_28.scaled(w,h,Qt.KeepAspectRatio))

    pix_29 = QPixmap(os.path.join(path,"asset/C_no.png"))
    self.ui.imageLabel_29.setPixmap(pix_29.scaled(w,h,Qt.KeepAspectRatio))

    pix_30 = QPixmap(os.path.join(path,"asset/Z_yes.png"))
    self.ui.imageLabel_30.setPixmap(pix_30.scaled(w,h,Qt.KeepAspectRatio))

    pix_31 = QPixmap(os.path.join(path,"asset/Z_no.png"))
    self.ui.imageLabel_31.setPixmap(pix_31.scaled(w,h,Qt.KeepAspectRatio))

    pix_32 = QPixmap(os.path.join(path,"asset/P_yes.png"))
    self.ui.imageLabel_32.setPixmap(pix_32.scaled(w,h,Qt.KeepAspectRatio))

    pix_33 = QPixmap(os.path.join(path,"asset/P_no.png"))
    self.ui.imageLabel_33.setPixmap(pix_33.scaled(w,h,Qt.KeepAspectRatio))

    pix_34 = QPixmap(os.path.join(path,"asset/OP_yes.png"))
    self.ui.imageLabel_34.setPixmap(pix_34.scaled(w,h,Qt.KeepAspectRatio))

    pix_35 = QPixmap(os.path.join(path,"asset/OP_no.png"))
    self.ui.imageLabel_35.setPixmap(pix_35.scaled(w,h,Qt.KeepAspectRatio))

    pix_36 = QPixmap(os.path.join(path,"asset/SAF_activated.png"))
    self.ui.imageLabel_36.setPixmap(pix_36.scaled(w,h,Qt.KeepAspectRatio))

    pix_37 = QPixmap(os.path.join(path,"asset/OD_PC.png"))
    self.ui.imageLabel_37.setPixmap(pix_37.scaled(w,h,Qt.KeepAspectRatio))

    pix_38 = QPixmap(os.path.join(path,"asset/specialPath.png"))
    self.ui.imageLabel_38.setPixmap(pix_38.scaled(w,h,Qt.KeepAspectRatio))

    pix_39 = QPixmap(os.path.join(path,"asset/killLPC.png"))
    self.ui.imageLabel_39.setPixmap(pix_39.scaled(w,h,Qt.KeepAspectRatio))

    pix_40 = QPixmap(os.path.join(path,"asset/R0_AS.png"))
    self.ui.imageLabel_40.setPixmap(pix_40.scaled(w,h,Qt.KeepAspectRatio))

    rect = QRect(QPoint(20,0),QSize(620,685))

    self.ui.imageLabel_Source.setGeometry(rect)
    self.ui.imageLabel.setGeometry(rect)
    self.ui.imageLabel_2.setGeometry(rect)
    self.ui.imageLabel_3.setGeometry(rect)
    self.ui.imageLabel_4.setGeometry(rect)
    self.ui.imageLabel_5.setGeometry(rect)
    self.ui.imageLabel_6.setGeometry(rect)
    self.ui.imageLabel_7.setGeometry(rect)
    self.ui.imageLabel_8.setGeometry(rect)
    self.ui.imageLabel_9.setGeometry(rect)
    self.ui.imageLabel_10.setGeometry(rect)
    self.ui.imageLabel_11.setGeometry(rect)
    self.ui.imageLabel_12.setGeometry(rect)
    self.ui.imageLabel_13.setGeometry(rect)
    self.ui.imageLabel_14.setGeometry(rect)
    self.ui.imageLabel_15.setGeometry(rect)
    self.ui.imageLabel_16.setGeometry(rect)
    self.ui.imageLabel_17.setGeometry(rect)
    self.ui.imageLabel_18.setGeometry(rect)
    self.ui.imageLabel_19.setGeometry(rect)
    self.ui.imageLabel_20.setGeometry(rect)
    self.ui.imageLabel_21.setGeometry(rect)
    self.ui.imageLabel_22.setGeometry(rect)
    self.ui.imageLabel_23.setGeometry(rect)
    self.ui.imageLabel_24.setGeometry(rect)
    # self.ui.imageLabel_25.setGeometry(rect)
    self.ui.imageLabel_26.setGeometry(rect)
    self.ui.imageLabel_27.setGeometry(rect)
    self.ui.imageLabel_28.setGeometry(rect)
    self.ui.imageLabel_29.setGeometry(rect)
    self.ui.imageLabel_30.setGeometry(rect)
    self.ui.imageLabel_31.setGeometry(rect)
    self.ui.imageLabel_32.setGeometry(rect)
    self.ui.imageLabel_33.setGeometry(rect)
    self.ui.imageLabel_34.setGeometry(rect)
    self.ui.imageLabel_35.setGeometry(rect)
    self.ui.imageLabel_36.setGeometry(rect)
    self.ui.imageLabel_37.setGeometry(rect)
    self.ui.imageLabel_38.setGeometry(rect)
    self.ui.imageLabel_39.setGeometry(rect)
    self.ui.imageLabel_40.setGeometry(rect)

    self.stack = self.ui.tableWidget_2
    self.stack.setRowCount(1)
    self.stack.setColumnCount(1)

    self.stack.verticalHeader().setVisible(False)

    headerx = QTableWidgetItem()
    headerx.setText("TOS")
    self.stack.setHorizontalHeaderItem(0,headerx)

    self.table= self.ui.tableWidget
    self.table.setRowCount(256)
    self.table.setColumnCount(2)
    self.table.verticalHeader().setVisible(False)

    header2 = QTableWidgetItem()
    header2.setText("Value")
    self.table.setHorizontalHeaderItem(1,header2)

    header1 = QTableWidgetItem()
    header1.setText("Address")
    self.table.setHorizontalHeaderItem(0,header1)

    rect2 = QRect(QPoint(0,0),QSize(self.width(),self.height()))
    self.ui.scrollArea.setWidget(self.ui.logBox)
    self.ui.PC_value.setText("0")
    self.ui.SP_value.setText("FF")
    self.ui.IR_value.setText("Opcode : XX")
    self.ui.OR_value.setText("XX")
    self.ui.Instruction_value.setText("Instruction : XX")
    self.ui.run10ClockCycle.setVisible(True)

  def on_nextStage_clicked(self):
    arch = self.inst_emulator.TraceVector[self.trace_index][0]
    march = self.inst_emulator.TraceVector[self.trace_index][1]

    controlBits = march.ControlBits
    PC = arch.PC
    SP = arch.SP
    OR = march.OperandRegister
    IR_reg = march.InstructionRegister
    flagArray = arch.Flags
    registerArray = arch.Registers
    memoryContents = arch.Memory
    state = march.State

    #Calculate from Regfile diff
    currentNew = -1

    # One Line of State has been read
    # if(controlBits[0] and (not controlBits[1]) and controlBits[2] and (not controlBits[3]) and (not controlBits[4]) and controlBits[5] and (not controlBits[6]) and (not controlBits[7]) and (not controlBits[8]) and (not controlBits[9]) and controlBits[10] and (not controlBits[11]) and (not controlBits[12]) and (not controlBits[13]) and (not controlBits[14]) and (not controlBits[15]) and (not controlBits[16]) and (not controlBits[17]) and (not controlBits[18]) and (not controlBits[19]) and (not controlBits[20]) and (not controlBits[21]))
    #     executionState = 0
        # 1 0 1 0
        # 0 1 0 0
        # 0 0 1 0
        # 0 0 0 0
        # 0 0 0 0
        # 0 0
    self.ui.state.setText(hex(state))
    self.ui.PC_value.setText(hex(PC))
    self.ui.SP_value.setText(hex((SP+256 - 1)%256))
    self.ui.OR_value.setText(hex(OR))
    self.ui.IR_value.setText("Opcode : "+hex(IR_reg))

    output = prettyPrintOpDecode(IR_reg)

    redColor = QPalette()
    redColor.setColor(QPalette.WindowText,Qt.red)

    blackColor = QPalette()
    blackColor.setColor(QPalette.WindowText,Qt.black)

    self.ui.Instruction_value.setText("Instruction : "+output)

    self.ui.R0_value.setPalette(blackColor)
    self.ui.R0_value.setText(hex(registerArray[0]))
    self.ui.R1_value.setPalette(blackColor)
    self.ui.R1_value.setText(hex(registerArray[1]))
    self.ui.R2_value.setPalette(blackColor)
    self.ui.R2_value.setText(hex(registerArray[2]))
    self.ui.R3_value.setPalette(blackColor)
    self.ui.R3_value.setText(hex(registerArray[3]))
    self.ui.R4_value.setPalette(blackColor)
    self.ui.R4_value.setText(hex(registerArray[4]))
    self.ui.R5_value.setPalette(blackColor)
    self.ui.R5_value.setText(hex(registerArray[5]))
    self.ui.R6_value.setPalette(blackColor)
    self.ui.R6_value.setText(hex(registerArray[6]))
    self.ui.R7_value.setPalette(blackColor)
    self.ui.R7_value.setText(hex(registerArray[7]))

    #     switch(currentNew){
    #     case 0:
    #         ui.R0_value.setPalette(redColor)
    #         break
    #     case 1:
    #         ui.R1_value.setPalette(redColor)
    #         break
    #     case 2:
    #         ui.R2_value.setPalette(redColor)
    #         break
    #     case 3:
    #         ui.R3_value.setPalette(redColor)
    #         break
    #     case 4:
    #         ui.R4_value.setPalette(redColor)
    #         break
    #     case 5:
    #         ui.R5_value.setPalette(redColor)
    #         break
    #     case 6:
    #         ui.R6_value.setPalette(redColor)
    #         break
    #     case 7:
    #         ui.R7_value.setPalette(redColor)
    #         break
    #     default:
    #         break
    #     }


    #     sprintf(tempString,"%X",flagArray[0])
    # //    ui.R0_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[1])
    # //    ui.R1_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[2])
    # //    ui.R2_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[3])
    # //    ui.R3_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[4])
    # //    ui.R4_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[5])
    # //    ui.R5_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[6])
    # //    ui.R6_value_2.setText(tempString)
    #     sprintf(tempString,"%X",flagArray[7])
    # //    ui.R7_value_2.setText(tempString)


    self.ui.imageLabel.setVisible(controlBits[0])
    self.ui.imageLabel_2.setVisible(controlBits[1])
    self.ui.imageLabel_3.setVisible(controlBits[2])
    self.ui.imageLabel_4.setVisible(controlBits[3])
    self.ui.imageLabel_5.setVisible(controlBits[4])
    self.ui.imageLabel_6.setVisible(controlBits[5])
    self.ui.imageLabel_7.setVisible(controlBits[6])
    self.ui.imageLabel_39.setVisible(controlBits[6])
    self.ui.imageLabel_8.setVisible(controlBits[7])
    self.ui.imageLabel_9.setVisible(controlBits[8])
    self.ui.imageLabel_10.setVisible(controlBits[9])
    self.ui.imageLabel_11.setVisible(controlBits[10])
    self.ui.imageLabel_12.setVisible(controlBits[11])
    self.ui.imageLabel_13.setVisible(controlBits[12])
    self.ui.imageLabel_14.setVisible(controlBits[13])
    self.ui.imageLabel_15.setVisible(controlBits[14])
    self.ui.imageLabel_16.setVisible(controlBits[15])
    self.ui.imageLabel_17.setVisible(controlBits[16])
    self.ui.imageLabel_18.setVisible(controlBits[17])
    self.ui.imageLabel_19.setVisible(controlBits[18])
    self.ui.imageLabel_20.setVisible(controlBits[19])
    self.ui.imageLabel_21.setVisible(controlBits[20])
    self.ui.imageLabel_22.setVisible(controlBits[21])
    self.ui.imageLabel_23.setVisible(controlBits[0] or controlBits[4] or controlBits[11] or controlBits[13] or controlBits[15] or controlBits[19] or controlBits[21])
    self.ui.imageLabel_24.setVisible(controlBits[8] and controlBits[9])
    self.ui.imageLabel_27.setVisible(controlBits[18])
    self.ui.imageLabel_26.setVisible(controlBits[20])
    self.ui.imageLabel_35.setVisible(controlBits[17])

    self.ui.imageLabel_28.setVisible(flagArray[2])
    self.ui.imageLabel_29.setVisible(flagArray[3])
    self.ui.imageLabel_30.setVisible(flagArray[0])
    self.ui.imageLabel_31.setVisible(flagArray[1])
    self.ui.imageLabel_32.setVisible(flagArray[4])
    self.ui.imageLabel_33.setVisible(flagArray[5])
    self.ui.imageLabel_34.setVisible(flagArray[6])
    self.ui.imageLabel_35.setVisible(flagArray[7])
    self.ui.imageLabel_38.setVisible(controlBits[5] and controlBits[6])
    # //    ui.imageLabel_37.setVisible(controlBits[5] and controlBits[6])

    self.ui.imageLabel_40.setVisible((not controlBits[3]) and (not controlBits[2]) and (controlBits[0] or controlBits[1]))
    if(controlBits[5] and controlBits[6]):
      self.ui.imageLabel_7.setVisible(False)

    self.ui.imageLabel_37.setVisible(controlBits[6] and controlBits[7])
    self.ui.imageLabel_36.setVisible(controlBits[17])
    if((controlBits[18] and controlBits[20]) or controlBits[17]):
      self.ui.imageLabel_19.setVisible(False)
      self.ui.imageLabel_21.setVisible(False)
    self.ui.OR_value.setVisible(False)
    self.ui.label_2.setVisible(False)

    if(controlBits[15] or controlBits[16] or controlBits[17] or (controlBits[5] and controlBits[6])):
      self.ui.OR_value.setVisible(True)
      self.ui.label_2.setVisible(True)
    
    for j in range(0,255):
      self.table.setItem(j, 0, QTableWidgetItem(hex(j)))
      self.table.setItem(j, 1, QTableWidgetItem(hex(memoryContents[j])))

    localTOS = (SP+256 - 1)%256
    heightOfStack = 255 - localTOS
    if(heightOfStack < 0):
      heightOfStack = 0

    self.stack.setColumnCount(1)
    self.stack.setRowCount(heightOfStack)
    for iterator in range(0,heightOfStack):
      self.stack.setItem(iterator, 0, QTableWidgetItem(hex(memoryContents[255-heightOfStack+iterator+1])))

    if(self.trace_index == 0):
      self.ui.Instruction_value.setText("Instruction : XX")
      self.ui.R0_value.setText("XX")
      self.ui.R1_value.setText("XX")
      self.ui.R2_value.setText("XX")
      self.ui.R3_value.setText("XX")
      self.ui.R4_value.setText("XX")
      self.ui.R5_value.setText("XX")
      self.ui.R6_value.setText("XX")
      self.ui.R7_value.setText("XX")

    self.trace_index = self.trace_index + 1
    self.inst_emulator.step_n(1)

  def on_pushButton_clicked(self):
    self.ui.logBox.setText("Compiling source files\n")
    try:
      self.last_compiled_mem,self.log_string = self.inst_assembler.compile(self.ui.plainTextEdit.toPlainText())
    except:
      self.ui.logBox.setText("Syntax Error\n")
      return

    self.inst_emulator.end_emulation()
    # print("Reset")
    self.inst_emulator.load_memory(self.last_compiled_mem)
    self.inst_emulator.start()
    self.inst_emulator.step_n(1)
    self.trace_index = 0
    self.on_nextStage_clicked()
    # self.inst_emulator.step_n(10)

    # self.ui.pushButton.setEnabled(False)
    # self.ui.pushButton_2.setEnabled(False)
    # self.ui.reset.setEnabled(False)
    # self.ui.nextStage.setEnabled(False)
    self.ui.logBox.setText(self.log_string)


    # self.on_reset_clicked()
    # self.ui.pushButton.setEnabled(True)
    # self.ui.pushButton_2.setEnabled(True)
    # self.ui.reset.setEnabled(True)
    # self.ui.nextStage.setEnabled(True)

  def on_pushButton_2_clicked(self):
    self.ui.logBox.setText(self.log_string)

  def on_reset_clicked(self):
    self.inst_emulator.end_emulation()
    # print("Reset")
    self.inst_emulator.load_memory(self.last_compiled_mem)
    self.inst_emulator.start()
    self.inst_emulator.step_n(1)
    self.trace_index = 0
    self.on_nextStage_clicked()

  def on_run10ClockCycle_clicked(self):
    for i in range(10):
      self.on_nextStage_clicked()
#### End of GUI Section



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../Emulator/RNBIP Qt Source/GUI_TEST/mainwidow.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWidow(object):
    def setupUi(self, MainWidow):
        MainWidow.setObjectName("MainWidow")
        MainWidow.resize(1366, 721)
        label_font = QtGui.QFont()
        label_font.setPointSize(10)
        label_font.setBold(False)
        label_font.setWeight(75)
        self.centralWidget = QtWidgets.QWidget(MainWidow)
        self.centralWidget.setMinimumSize(QtCore.QSize(1110, 0))
        self.centralWidget.setObjectName("centralWidget")
        self.imageLabel_Source = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_Source.setGeometry(QtCore.QRect(-190, 240, 701, 671))
        self.imageLabel_Source.setText("")
        self.imageLabel_Source.setObjectName("imageLabel_Source")
        self.layoutWidget = QtWidgets.QWidget(self.centralWidget)
        self.layoutWidget.setGeometry(QtCore.QRect(20, 160, 151, 91))
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.nextStage = QtWidgets.QPushButton(self.centralWidget)
        self.nextStage.setFont(label_font)
        self.nextStage.setGeometry(QtCore.QRect(650, 540, 180, 51))
        self.nextStage.setAutoFillBackground(False)
        self.nextStage.setObjectName("nextStage")
        self.imageLabel_2 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_2.setGeometry(QtCore.QRect(-120, 280, 701, 671))
        self.imageLabel_2.setText("")
        self.imageLabel_2.setObjectName("imageLabel_2")
        self.imageLabel_3 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_3.setGeometry(QtCore.QRect(-160, 270, 701, 671))
        self.imageLabel_3.setText("")
        self.imageLabel_3.setObjectName("imageLabel_3")
        self.imageLabel_4 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_4.setGeometry(QtCore.QRect(-270, 290, 701, 671))
        self.imageLabel_4.setText("")
        self.imageLabel_4.setObjectName("imageLabel_4")
        self.imageLabel_5 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_5.setGeometry(QtCore.QRect(-210, 280, 701, 671))
        self.imageLabel_5.setText("")
        self.imageLabel_5.setObjectName("imageLabel_5")
        self.imageLabel_6 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_6.setGeometry(QtCore.QRect(-190, 280, 701, 671))
        self.imageLabel_6.setText("")
        self.imageLabel_6.setObjectName("imageLabel_6")
        self.imageLabel_7 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_7.setGeometry(QtCore.QRect(-200, 230, 701, 671))
        self.imageLabel_7.setText("")
        self.imageLabel_7.setObjectName("imageLabel_7")
        self.imageLabel_8 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_8.setGeometry(QtCore.QRect(-180, 290, 701, 671))
        self.imageLabel_8.setText("")
        self.imageLabel_8.setObjectName("imageLabel_8")
        self.imageLabel_9 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_9.setGeometry(QtCore.QRect(-250, 220, 701, 671))
        self.imageLabel_9.setText("")
        self.imageLabel_9.setObjectName("imageLabel_9")
        self.imageLabel_10 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_10.setGeometry(QtCore.QRect(-180, 280, 701, 671))
        self.imageLabel_10.setText("")
        self.imageLabel_10.setObjectName("imageLabel_10")
        self.imageLabel_11 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_11.setGeometry(QtCore.QRect(-190, 280, 701, 671))
        self.imageLabel_11.setText("")
        self.imageLabel_11.setObjectName("imageLabel_11")
        self.imageLabel_12 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_12.setGeometry(QtCore.QRect(-200, 290, 701, 671))
        self.imageLabel_12.setText("")
        self.imageLabel_12.setObjectName("imageLabel_12")
        self.imageLabel_13 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_13.setGeometry(QtCore.QRect(-190, 250, 701, 671))
        self.imageLabel_13.setText("")
        self.imageLabel_13.setObjectName("imageLabel_13")
        self.imageLabel_14 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_14.setGeometry(QtCore.QRect(-230, 250, 701, 671))
        self.imageLabel_14.setText("")
        self.imageLabel_14.setObjectName("imageLabel_14")
        self.imageLabel_15 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_15.setGeometry(QtCore.QRect(-320, 160, 701, 671))
        self.imageLabel_15.setText("")
        self.imageLabel_15.setObjectName("imageLabel_15")
        self.imageLabel_16 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_16.setGeometry(QtCore.QRect(880, 390, 701, 671))
        self.imageLabel_16.setText("")
        self.imageLabel_16.setObjectName("imageLabel_16")
        self.imageLabel_17 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_17.setGeometry(QtCore.QRect(810, 400, 701, 671))
        self.imageLabel_17.setText("")
        self.imageLabel_17.setObjectName("imageLabel_17")
        self.imageLabel_18 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_18.setGeometry(QtCore.QRect(960, 400, 701, 671))
        self.imageLabel_18.setText("")
        self.imageLabel_18.setObjectName("imageLabel_18")
        self.imageLabel_19 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_19.setGeometry(QtCore.QRect(820, 400, 701, 671))
        self.imageLabel_19.setText("")
        self.imageLabel_19.setObjectName("imageLabel_19")
        self.imageLabel_20 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_20.setGeometry(QtCore.QRect(830, 400, 701, 671))
        self.imageLabel_20.setText("")
        self.imageLabel_20.setObjectName("imageLabel_20")
        self.imageLabel_21 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_21.setGeometry(QtCore.QRect(-140, 410, 701, 671))
        self.imageLabel_21.setText("")
        self.imageLabel_21.setObjectName("imageLabel_21")
        self.imageLabel_22 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_22.setGeometry(QtCore.QRect(-200, 30, 701, 671))
        self.imageLabel_22.setText("")
        self.imageLabel_22.setObjectName("imageLabel_22")
        self.imageLabel = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel.setGeometry(QtCore.QRect(40, 40, 5, 13))
        self.imageLabel.setText("")
        self.imageLabel.setObjectName("imageLabel")
        self.plainTextEdit = QtWidgets.QPlainTextEdit(self.centralWidget)
        self.plainTextEdit.setGeometry(QtCore.QRect(920, 60, 421, 221))
        font = QtGui.QFont()
        font.setFamily("Courier New")
        self.plainTextEdit.setFont(font)
        self.plainTextEdit.setAutoFillBackground(False)
        self.plainTextEdit.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.plainTextEdit.setTabChangesFocus(False)
        self.plainTextEdit.setBackgroundVisible(False)
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.pushButton = QtWidgets.QPushButton(self.centralWidget)
        self.pushButton.setGeometry(QtCore.QRect(1040, 290, 81, 23))
        self.pushButton.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.pushButton.setAutoDefault(False)
        self.pushButton.setDefault(False)
        self.pushButton.setFlat(False)
        self.pushButton.setObjectName("pushButton")
        self.logBox = QtWidgets.QTextBrowser(self.centralWidget)
        self.logBox.setGeometry(QtCore.QRect(920, 330, 421, 192))
        self.logBox.viewport().setProperty("cursor", QtGui.QCursor(QtCore.Qt.IBeamCursor))
        self.logBox.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.logBox.setFrameShadow(QtWidgets.QFrame.Plain)
        self.logBox.setObjectName("logBox")
        self.scrollArea = QtWidgets.QScrollArea(self.centralWidget)
        self.scrollArea.setGeometry(QtCore.QRect(919, 330, 421, 91))
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 421, 91))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayoutWidget = QtWidgets.QWidget(self.centralWidget)
        self.gridLayoutWidget.setGeometry(QtCore.QRect(630, 260, 201, 268))
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.gridLayout.setContentsMargins(11, 11, 11, 11)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.R2_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R2_value.setFont(font)
        self.R2_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R2_value.setObjectName("R2_value")
        self.gridLayout.addWidget(self.R2_value, 4, 2, 1, 1)
        self.R2 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R2.setFont(font)
        self.R2.setAlignment(QtCore.Qt.AlignCenter)
        self.R2.setObjectName("R2")
        self.gridLayout.addWidget(self.R2, 4, 0, 1, 1)
        self.line = QtWidgets.QFrame(self.gridLayoutWidget)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 2, 1, 8, 1)
        self.label_8 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(16)
        self.label_8.setFont(font)
        self.label_8.setAlignment(QtCore.Qt.AlignCenter)
        self.label_8.setObjectName("label_8")
        self.label_8.setFont(label_font)
        self.gridLayout.addWidget(self.label_8, 0, 0, 1, 3)
        self.R3_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R3_value.setFont(font)
        self.R3_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R3_value.setObjectName("R3_value")
        self.gridLayout.addWidget(self.R3_value, 5, 2, 1, 1)
        self.R0_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R0_value.setFont(font)
        self.R0_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R0_value.setObjectName("R0_value")
        self.gridLayout.addWidget(self.R0_value, 2, 2, 1, 1)
        self.R1_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R1_value.setFont(font)
        self.R1_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R1_value.setObjectName("R1_value")
        self.gridLayout.addWidget(self.R1_value, 3, 2, 1, 1)
        self.R4_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R4_value.setFont(font)
        self.R4_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R4_value.setObjectName("R4_value")
        self.gridLayout.addWidget(self.R4_value, 6, 2, 1, 1)
        self.R4 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R4.setFont(font)
        self.R4.setAlignment(QtCore.Qt.AlignCenter)
        self.R4.setObjectName("R4")
        self.gridLayout.addWidget(self.R4, 6, 0, 1, 1)
        self.R0 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R0.setFont(font)
        self.R0.setAlignment(QtCore.Qt.AlignCenter)
        self.R0.setObjectName("R0")
        self.gridLayout.addWidget(self.R0, 2, 0, 1, 1)
        self.R1 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R1.setFont(font)
        self.R1.setAlignment(QtCore.Qt.AlignCenter)
        self.R1.setObjectName("R1")
        self.gridLayout.addWidget(self.R1, 3, 0, 1, 1)
        self.R7_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R7_value.setFont(font)
        self.R7_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R7_value.setObjectName("R7_value")
        self.gridLayout.addWidget(self.R7_value, 9, 2, 1, 1)
        self.R6 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R6.setFont(font)
        self.R6.setAlignment(QtCore.Qt.AlignCenter)
        self.R6.setObjectName("R6")
        self.gridLayout.addWidget(self.R6, 8, 0, 1, 1)
        self.R6_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R6_value.setFont(font)
        self.R6_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R6_value.setObjectName("R6_value")
        self.gridLayout.addWidget(self.R6_value, 8, 2, 1, 1)
        self.R7 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R7.setFont(font)
        self.R7.setAlignment(QtCore.Qt.AlignCenter)
        self.R7.setObjectName("R7")
        self.gridLayout.addWidget(self.R7, 9, 0, 1, 1)
        self.R5 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R5.setFont(font)
        self.R5.setAlignment(QtCore.Qt.AlignCenter)
        self.R5.setObjectName("R5")
        self.gridLayout.addWidget(self.R5, 7, 0, 1, 1)
        self.R5_value = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.R5_value.setFont(font)
        self.R5_value.setAlignment(QtCore.Qt.AlignCenter)
        self.R5_value.setObjectName("R5_value")
        self.gridLayout.addWidget(self.R5_value, 7, 2, 1, 1)
        self.R3 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.R3.setFont(font)
        self.R3.setAlignment(QtCore.Qt.AlignCenter)
        self.R3.setObjectName("R3")
        self.gridLayout.addWidget(self.R3, 5, 0, 1, 1)
        self.line_6 = QtWidgets.QFrame(self.gridLayoutWidget)
        self.line_6.setLineWidth(2)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.gridLayout.addWidget(self.line_6, 1, 0, 1, 3)
        self.formLayoutWidget = QtWidgets.QWidget(self.centralWidget)
        self.formLayoutWidget.setGeometry(QtCore.QRect(0, 10, 1371, 120))
        self.formLayoutWidget.setObjectName("formLayoutWidget")
        self.formLayout = QtWidgets.QFormLayout(self.formLayoutWidget)
        self.formLayout.setContentsMargins(11, 11, 11, 11)
        self.formLayout.setSpacing(6)
        self.formLayout.setObjectName("formLayout")
        self.Instruction_value = QtWidgets.QLabel(self.formLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(20)
        self.Instruction_value.setFont(font)
        self.Instruction_value.setAlignment(QtCore.Qt.AlignCenter)
        self.Instruction_value.setObjectName("Instruction_value")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.Instruction_value)
        self.IR_value = QtWidgets.QLabel(self.formLayoutWidget)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(14)
        self.IR_value.setFont(font)
        self.IR_value.setAlignment(QtCore.Qt.AlignCenter)
        self.IR_value.setObjectName("IR_value")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.IR_value)
        self.IR_value.raise_()
        self.Instruction_value.raise_()
        self.imageLabel_23 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_23.setGeometry(QtCore.QRect(270, 640, 701, 671))
        self.imageLabel_23.setText("")
        self.imageLabel_23.setObjectName("imageLabel_23")
        self.imageLabel_24 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_24.setGeometry(QtCore.QRect(-330, 300, 701, 671))
        self.imageLabel_24.setObjectName("imageLabel_24")
        self.pushButton_2 = QtWidgets.QPushButton(self.centralWidget)
        self.pushButton_2.setGeometry(QtCore.QRect(1130, 290, 81, 23))
        self.pushButton_2.setObjectName("pushButton_2")
        self.tableWidget = QtWidgets.QTableWidget(self.centralWidget)
        self.tableWidget.setGeometry(QtCore.QRect(1140, 470, 201, 192))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tableWidget.sizePolicy().hasHeightForWidth())
        self.tableWidget.setSizePolicy(sizePolicy)
        self.tableWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.tableWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.tableWidget.setAutoScroll(True)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setDragEnabled(True)
        self.tableWidget.setDragDropMode(QtWidgets.QAbstractItemView.NoDragDrop)
        self.tableWidget.setDefaultDropAction(QtCore.Qt.IgnoreAction)
        self.tableWidget.setAlternatingRowColors(True)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(0)
        self.tableWidget.setRowCount(0)
        self.imageLabel_25 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_25.setGeometry(QtCore.QRect(1180, 260, 701, 671))
        self.imageLabel_25.setText("")
        self.imageLabel_25.setObjectName("imageLabel_25")
        self.imageLabel_26 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_26.setGeometry(QtCore.QRect(-210, 280, 701, 671))
        self.imageLabel_26.setText("")
        self.imageLabel_26.setObjectName("imageLabel_26")
        self.imageLabel_27 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_27.setGeometry(QtCore.QRect(1170, 280, 701, 671))
        self.imageLabel_27.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_27.setText("")
        self.imageLabel_27.setObjectName("imageLabel_27")
        self.gridLayoutWidget_3 = QtWidgets.QWidget(self.centralWidget)
        self.gridLayoutWidget_3.setGeometry(QtCore.QRect(610, 150, 241, 80))
        self.gridLayoutWidget_3.setObjectName("gridLayoutWidget_3")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.gridLayoutWidget_3)
        self.gridLayout_3.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_3.setSpacing(6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.PC_value = QtWidgets.QLabel(self.gridLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.PC_value.setFont(font)
        self.PC_value.setAlignment(QtCore.Qt.AlignCenter)
        self.PC_value.setObjectName("PC_value")
        self.gridLayout_3.addWidget(self.PC_value, 1, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.gridLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 0, 1, 1, 1)
        self.SP_value = QtWidgets.QLabel(self.gridLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.SP_value.setFont(font)
        self.SP_value.setAlignment(QtCore.Qt.AlignCenter)
        self.SP_value.setObjectName("SP_value")
        self.gridLayout_3.addWidget(self.SP_value, 1, 1, 1, 1)
        self.PCLabel = QtWidgets.QLabel(self.gridLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.PCLabel.setFont(font)
        self.PCLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.PCLabel.setObjectName("PCLabel")
        self.gridLayout_3.addWidget(self.PCLabel, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.gridLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 0, 2, 1, 1)
        self.OR_value = QtWidgets.QLabel(self.gridLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.OR_value.setFont(font)
        self.OR_value.setAlignment(QtCore.Qt.AlignCenter)
        self.OR_value.setObjectName("OR_value")
        self.gridLayout_3.addWidget(self.OR_value, 1, 2, 1, 1)
        self.reset = QtWidgets.QPushButton(self.centralWidget)
        self.reset.setGeometry(QtCore.QRect(700, 610, 75, 23))
        self.reset.setObjectName("reset")
        self.imageLabel_28 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_28.setGeometry(QtCore.QRect(1210, 50, 701, 671))
        self.imageLabel_28.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_28.setText("")
        self.imageLabel_28.setObjectName("imageLabel_28")
        self.imageLabel_29 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_29.setGeometry(QtCore.QRect(1210, 50, 701, 671))
        self.imageLabel_29.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_29.setText("")
        self.imageLabel_29.setObjectName("imageLabel_29")
        self.imageLabel_30 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_30.setGeometry(QtCore.QRect(1210, 50, 701, 671))
        self.imageLabel_30.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_30.setText("")
        self.imageLabel_30.setObjectName("imageLabel_30")
        self.imageLabel_31 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_31.setGeometry(QtCore.QRect(1210, 50, 701, 671))
        self.imageLabel_31.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_31.setText("")
        self.imageLabel_31.setObjectName("imageLabel_31")
        self.imageLabel_32 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_32.setGeometry(QtCore.QRect(1210, 50, 701, 671))
        self.imageLabel_32.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_32.setText("")
        self.imageLabel_32.setObjectName("imageLabel_32")
        self.imageLabel_33 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_33.setGeometry(QtCore.QRect(1210, 50, 701, 671))
        self.imageLabel_33.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_33.setText("")
        self.imageLabel_33.setObjectName("imageLabel_33")
        self.imageLabel_34 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_34.setGeometry(QtCore.QRect(-150, 50, 701, 671))
        self.imageLabel_34.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_34.setText("")
        self.imageLabel_34.setObjectName("imageLabel_34")
        self.imageLabel_35 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_35.setGeometry(QtCore.QRect(-330, 50, 701, 671))
        self.imageLabel_35.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_35.setText("")
        self.imageLabel_35.setObjectName("imageLabel_35")
        self.imageLabel_36 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_36.setGeometry(QtCore.QRect(1240, 10, 701, 671))
        self.imageLabel_36.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_36.setText("")
        self.imageLabel_36.setObjectName("imageLabel_36")
        self.tableWidget_2 = QtWidgets.QTableWidget(self.centralWidget)
        self.tableWidget_2.setGeometry(QtCore.QRect(920, 470, 101, 192))
        self.tableWidget_2.setLineWidth(1)
        self.tableWidget_2.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.tableWidget_2.setAutoScroll(True)
        self.tableWidget_2.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget_2.setDragEnabled(True)
        self.tableWidget_2.setDragDropMode(QtWidgets.QAbstractItemView.NoDragDrop)
        self.tableWidget_2.setDefaultDropAction(QtCore.Qt.IgnoreAction)
        self.tableWidget_2.setAlternatingRowColors(True)
        self.tableWidget_2.setObjectName("tableWidget_2")
        self.tableWidget_2.setColumnCount(0)
        self.tableWidget_2.setRowCount(0)
        self.imageLabel_37 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_37.setGeometry(QtCore.QRect(1000, 80, 701, 671))
        self.imageLabel_37.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_37.setText("")
        self.imageLabel_37.setObjectName("imageLabel_37")
        self.run10ClockCycle = QtWidgets.QPushButton(self.centralWidget)
        self.run10ClockCycle.setGeometry(QtCore.QRect(680, 640, 111, 51))
        self.run10ClockCycle.setObjectName("run10ClockCycle")
        self.imageLabel_38 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_38.setGeometry(QtCore.QRect(-160, 10, 701, 671))
        self.imageLabel_38.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_38.setText("")
        self.imageLabel_38.setObjectName("imageLabel_38")
        self.imageLabel_39 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_39.setGeometry(QtCore.QRect(-150, 10, 701, 671))
        self.imageLabel_39.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_39.setText("")
        self.imageLabel_39.setObjectName("imageLabel_39")
        self.imageLabel_40 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_40.setGeometry(QtCore.QRect(890, 220, 701, 671))
        self.imageLabel_40.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_40.setText("")
        self.imageLabel_40.setObjectName("imageLabel_40")
        self.imageLabel_41 = QtWidgets.QLabel(self.centralWidget)
        self.imageLabel_41.setGeometry(QtCore.QRect(1310, 540, 701, 671))
        self.imageLabel_41.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.imageLabel_41.setText("")
        self.imageLabel_41.setObjectName("imageLabel_41")
        self.label_3 = QtWidgets.QLabel(self.centralWidget)
        self.label_3.setGeometry(QtCore.QRect(920, 40, 80, 13))
        self.label_3.setObjectName("label_3")
        self.label_3.setFont(label_font)
        self.label_4 = QtWidgets.QLabel(self.centralWidget)
        self.label_4.setGeometry(QtCore.QRect(920, 310, 47, 13))
        self.label_4.setObjectName("label_4")
        self.label_4.setFont(label_font)
        self.state_3 = QtWidgets.QLabel(self.centralWidget)
        self.state_3.setGeometry(QtCore.QRect(40, 130, 1363, 23))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.state_3.setFont(font)
        self.state_3.setAlignment(QtCore.Qt.AlignCenter)
        self.state_3.setObjectName("state_3")
        self.state = QtWidgets.QLabel(self.centralWidget)
        self.state.setGeometry(QtCore.QRect(70, 130, 1363, 23))
        font = QtGui.QFont()
        font.setPointSize(14)
        self.state.setFont(font)
        self.state.setAlignment(QtCore.Qt.AlignCenter)
        self.state.setObjectName("state")
        self.imageLabel_37.raise_()
        self.imageLabel_Source.raise_()
        self.layoutWidget.raise_()
        self.imageLabel_2.raise_()
        self.imageLabel_3.raise_()
        self.imageLabel_4.raise_()
        self.imageLabel_5.raise_()
        self.imageLabel_6.raise_()
        self.imageLabel_7.raise_()
        self.imageLabel_8.raise_()
        self.imageLabel_9.raise_()
        self.imageLabel_10.raise_()
        self.imageLabel_11.raise_()
        self.imageLabel_12.raise_()
        self.imageLabel_13.raise_()
        self.imageLabel_14.raise_()
        self.imageLabel_15.raise_()
        self.imageLabel_16.raise_()
        self.imageLabel_17.raise_()
        self.imageLabel_18.raise_()
        self.imageLabel_19.raise_()
        self.imageLabel_20.raise_()
        self.imageLabel_21.raise_()
        self.imageLabel_22.raise_()
        self.imageLabel.raise_()
        self.nextStage.raise_()
        self.pushButton.raise_()
        self.logBox.raise_()
        self.scrollArea.raise_()
        self.gridLayoutWidget.raise_()
        self.formLayoutWidget.raise_()
        self.imageLabel_23.raise_()
        self.imageLabel_24.raise_()
        self.pushButton_2.raise_()
        self.imageLabel_25.raise_()
        self.imageLabel_27.raise_()
        self.imageLabel_26.raise_()
        self.gridLayoutWidget_3.raise_()
        self.reset.raise_()
        self.imageLabel_28.raise_()
        self.imageLabel_29.raise_()
        self.imageLabel_31.raise_()
        self.imageLabel_32.raise_()
        self.imageLabel_33.raise_()
        self.imageLabel_34.raise_()
        self.imageLabel_35.raise_()
        self.imageLabel_36.raise_()
        self.tableWidget_2.raise_()
        self.plainTextEdit.raise_()
        self.tableWidget.raise_()
        self.run10ClockCycle.raise_()
        self.imageLabel_38.raise_()
        self.imageLabel_39.raise_()
        self.imageLabel_40.raise_()
        self.imageLabel_41.raise_()
        self.label_3.raise_()
        self.label_4.raise_()
        self.imageLabel_30.raise_()
        self.state_3.raise_()
        self.state.raise_()
        MainWidow.setCentralWidget(self.centralWidget)
        self.menuBar = QtWidgets.QMenuBar(MainWidow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 1366, 21))
        self.menuBar.setObjectName("menuBar")
        MainWidow.setMenuBar(self.menuBar)
        self.statusBar = QtWidgets.QStatusBar(MainWidow)
        self.statusBar.setObjectName("statusBar")
        MainWidow.setStatusBar(self.statusBar)
        self.actionReset = QtWidgets.QAction(MainWidow)
        self.actionReset.setObjectName("actionReset")
        self.actionNext_Cycle = QtWidgets.QAction(MainWidow)
        self.actionNext_Cycle.setObjectName("actionNext_Cycle")
        self.actionExit = QtWidgets.QAction(MainWidow)
        self.actionExit.setObjectName("actionExit")
        self.ResetAction = QtWidgets.QAction(MainWidow)
        self.ResetAction.setObjectName("ResetAction")

        self.retranslateUi(MainWidow)
        # QtCore.QMetaObject.connectSlotsByName(MainWidow)

    def retranslateUi(self, MainWidow):
        _translate = QtCore.QCoreApplication.translate
        MainWidow.setWindowTitle(_translate("MainWidow", "MainWidow"))
        self.nextStage.setText(_translate("MainWidow", "Next Clock Cycle"))
        self.pushButton.setText(_translate("MainWidow", "Compile"))
        self.R2_value.setText(_translate("MainWidow", "XX"))
        self.R2.setText(_translate("MainWidow", "R2"))
        self.label_8.setText(_translate("MainWidow", "Register Array"))
        self.R3_value.setText(_translate("MainWidow", "XX"))
        self.R0_value.setText(_translate("MainWidow", "XX"))
        self.R1_value.setText(_translate("MainWidow", "XX"))
        self.R4_value.setText(_translate("MainWidow", "XX"))
        self.R4.setText(_translate("MainWidow", "R4"))
        self.R0.setText(_translate("MainWidow", "R0"))
        self.R1.setText(_translate("MainWidow", "R1"))
        self.R7_value.setText(_translate("MainWidow", "XX"))
        self.R6.setText(_translate("MainWidow", "R6"))
        self.R6_value.setText(_translate("MainWidow", "XX"))
        self.R7.setText(_translate("MainWidow", "R7"))
        self.R5.setText(_translate("MainWidow", "R5"))
        self.R5_value.setText(_translate("MainWidow", "XX"))
        self.R3.setText(_translate("MainWidow", "R3"))
        self.Instruction_value.setText(_translate("MainWidow", "TextLabel"))
        self.IR_value.setText(_translate("MainWidow", "TextLabel"))
        self.imageLabel_24.setText(_translate("MainWidow", "Zero"))
        self.pushButton_2.setText(_translate("MainWidow", "View Log"))
        self.PC_value.setText(_translate("MainWidow", "0"))
        self.label.setText(_translate("MainWidow", "SP"))
        self.SP_value.setText(_translate("MainWidow", "FF"))
        self.PCLabel.setText(_translate("MainWidow", "PC"))
        self.label_2.setText(_translate("MainWidow", "OR"))
        self.OR_value.setText(_translate("MainWidow", "XX"))
        self.reset.setText(_translate("MainWidow", "Reset"))
        self.run10ClockCycle.setText(_translate("MainWidow", "Run 10 Cycles"))
        self.label_3.setText(_translate("MainWidow", "EDITOR"))
        self.label_4.setText(_translate("MainWidow", "LOG"))
        self.state_3.setText(_translate("MainWidow", "State :    "))
        self.state.setText(_translate("MainWidow", "XX"))
        self.actionReset.setText(_translate("MainWidow", "Reset"))
        self.actionNext_Cycle.setText(_translate("MainWidow", "Next Cycle"))
        self.actionExit.setText(_translate("MainWidow", "Exit"))
        self.ResetAction.setText(_translate("MainWidow", "Reset"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWidow = QtWidgets.QMainWindow()
    ui = Ui_MainWidow()
    ui.setupUi(MainWidow)
    MainWidow.show()
    sys.exit(app.exec_())



import time
import logging
import random
import pdb
import os
import datetime;
import platform
import re
import sys

from rnbip_emulator.rnbip_isa import *

def log(x):
  flag=0;
  if len(str(x)) ==1:
    x = '0'+str(x)
  if len(str(x)) == 2:
    if str(x)[0] in ['1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','0'] and str(x)[1] in ['1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','0'] :
      return x;
    return("Error @ "+" \""+str(x)+"\" is not a recognised Literal\n")
  return("Error @ "+" \""+str(x)+"\" is not a recognised Instruction\n")

class rnbip_assembler():
  
  @staticmethod
  def file_len(fname):
    i=0
    with fname as f:
      for i, l in enumerate(f):
        pass
    return i + 1
  
  @staticmethod
  def tobin(x):
    binary = ''
    if len(x) == 2:
      for i in x:
        # print(i)
        binary = binary+Hex2Bin_LUT(i.strip())
    elif len(x)==1:
      x="0"+x
    for i in x:
      binary = binary+Hex2Bin_LUT(i.strip())
    return binary

  def compile(self,str_asm):
    list_of_od_operations=["MVI R","ADI R","SBI R","ACI R","SCI R","ANI R","ORI R","XRI R","JUD","CUD","JCD R","CCD R","JCA R","CCA R","RTC R"]
    for codes in list_of_od_operations:
      if "R" == codes[len(codes)-1]:
        for i in range(0,8):
          list_of_od_operations.append(codes+str(i))
    list_of_flags = ["Z","NZ","C","NC","P","N","OP","EP"]
    #print(os.path.dirname(__file__))
    counter = 0;
    flag=1;
    labels = [];
    temp_str = ""
    log_str = ""
    out_str = ""
    final_binary = ""
    for line in str_asm.splitlines():
      if "//" in line.strip():
        if "/" in line[0].strip():
          continue;
        else:
          line = line[0:line.index('//')];
          ##print("outside else JUD")
          if "JUD" in line:
            ##print("inside else JUD")
            t = line[[m.start() for m in re.finditer(r" ",line)][0]:]
            line = " JUD "+t
            ##print("this is it"+ line.strip())
          if "CUD" in line:
            t = line[[m.start() for m in re.finditer(r" ",line)][0]:]
            line = " CUD "+t
          if " " in line.strip():
            if len([m.start() for m in re.finditer(r" ",line)]) > 1:
              temp = line[[m.start() for m in re.finditer(r" ",line)][1]:]
              if temp.strip() in list_of_flags:
                temp = "0"+str(list_of_flags.index(temp.strip()))
              line = line[0:[m.start() for m in re.finditer(r" ",line)][1]]
              tline = assembly2IR_LUT(line.strip())
              if(tline == "CHECK_LITERAL"):
                line = log(line.strip())
              else:
                line = tline
              if line.strip() in list_of_od_operations:
                line = line + "\n" + temp
              else:
                line = log(line)+ "\n" + temp
        output = line.strip()
        if len(output) < 20:
          temp_str = temp_str + output+"\n"
        else:
          log_str = log_str + "Error at " + output
          sys.exit()
      elif line[0]=="":
        continue;
      else:
        ##print("outside else JUD")
        if "JUD " in line:
          ##print("inside else JUD")
          t = line[[m.start() for m in re.finditer(r" ",line)][0]:]
          line = " JUD "+t
          ##print("this is it"+ line.strip())
        if "CUD " in line:
            t = line[[m.start() for m in re.finditer(r" ",line)][0]:]
            line = " CUD "+t
        if " " in line.strip():
          if len([m.start() for m in re.finditer(r" ",line)]) > 1:
            #print("inside if")
            temp = line[[m.start() for m in re.finditer(r" ",line)][1]:]
            if temp.strip() in list_of_flags:
              temp = "0"+str(list_of_flags.index(temp.strip()))
            line = line[0:[m.start() for m in re.finditer(r" ",line)][1]]
            tline = assembly2IR_LUT(line.strip())
            if(tline == "CHECK_LITERAL"):
              line = log(line.strip())
            else:
              line = tline
            if line.strip() in list_of_od_operations:
              #print("inside od list")
              #print (line)
              line = line + "\n" + temp
            else:
              line = log(line)+ "\n" + temp
      if len(line) < 20:
        #print(line)
        temp_str = temp_str + line +"\n"
      else:
        log_str = log_str + "Error at " + line
        sys.exit()
    # print("done")

    for line in temp_str.splitlines():
      counter = counter+1;
      lab=0;
      if line[0]=="\n":
        continue
      if ":" in line.strip():
        labels.append((counter,line[0:line.index(':')]));
        # print(labels);
        if len(labels)!=0:
            for x in labels:
              if x[1] == line[line.index(':')+1:].strip():
                # print(x[0])
                toutput = assembly2IR_LUT(x[0]);
                if(toutput == "CHECK_LITERAL"):
                  output = log(x[0])
                else:
                  output = toutput
                # print(output);
                out_str = out_str + str(output)+"\n"
                break
        if line[line.index(':')+1] == "\n":
          continue
        else:
          is_inside = 0
          line = line[line.index(':')+1:].strip()
          if " " in line.strip():
            if len([m.start() for m in re.finditer(r" ",line)]) > 1:
              is_inside=1
              temp = line[[m.start() for m in re.finditer(r" ",line)][1]:]
              if temp.strip() in list_of_flags:
                temp = "0"+str(list_of_flags.index(temp.strip()))
              line = line[0:[m.start() for m in re.finditer(r" ",line)][1]]
              tline = assembly2IR_LUT(line)
              if(tline == "CHECK_LITERAL"):
                line = log(line)
              else:
                line = tline
              if line.strip() in list_of_od_operations:
                line = line.upper() + "\n" + temp
              else:
                line = log(line.upper())+ "\n" + temp
              # line = line.upper() + "\n" + temp
          output = line.strip()
          # output = line[line.index(':')+1:].strip().upper();
          # print(output);
          if is_inside == 1:
            if len(output)<20:
              out_str = out_str + output+"\n"
            else:
              sys.exit()
              log_str = log_str + "Error at " + output
          else:
            toutput = assembly2IR_LUT(line.strip().upper())
            if(toutput == "CHECK_LITERAL"):
              output = log(line.strip().upper())
            else:
              output = toutput
            if len(output)<20:
              out_str = out_str + output+"\n"
            else:
              sys.exit()
              log_str = log_str + "Error at " + output
      else:
        if len(labels)!=0:
            for x in labels:
              if x[1] == line.strip():
                # print(x[0])
                toutput = assembly2IR_LUT(x[0]);
                if(toutput == "CHECK_LITERAL"):
                  output = log(x[0])
                else:
                  output = toutput
                # print(output);
                out_str = out_str + str(output)+"\n"
                lab=1;
                break
        if lab==0:
          is_inside = 0;
          if " " in line.strip():
            if len([m.start() for m in re.finditer(r" ",line)]) > 1:
              # print("if")
              # print(line)
              is_inside = 1;
              temp = line[[m.start() for m in re.finditer(r" ",line)][1]:]
              if temp.strip() in list_of_flags:
                temp = "0"+str(list_of_flags.index(temp.strip()))
              line = line[0:[m.start() for m in re.finditer(r" ",line)][1]]
              tline = assembly2IR_LUT(line)
              if(tline == "CHECK_LITERAL"):
                line = log(line)
              else:
                line = tline
              if line.strip() in list_of_od_operations:
                line = line.upper() + "\n" + temp
              else:
                line = log(line.upper())+ "\n" + temp
          output = line.strip();
          #print(output);
          #print("hherererererer")
          if is_inside == 1:
            if len(output)<20:
              out_str = out_str + output+"\n"
            else:
              log_str = log_str + "Error at " + output
              sys.exit()
          else:
            #print("this is where it should be:"+line)
            tempx = 0
            output = IR2Binary_LUT(line.strip().upper())
            # print(output)
            if output.isdigit():
              tempx=1
            try:
              int(output, 16)
              tempx = 1
            except ValueError:
              tempx = 0
            if len(output)<20 and tempx==1 :
              out_str = out_str + output+"\n"
            else:
              log_str = log_str + "Error at " + output
              sys.exit()
    if flag == 1:
      #print(str(datetime.datetime.now()));
      log_str = log_str + str(datetime.datetime.now()) + " Compilation Successful...\n"

    #2ND Pass
    #ControlFile = open("cc.txt","w")
    for line in out_str.splitlines() :
      if "//" not in line.strip():
        output = IR2Binary_LUT(line.strip());
        # print(output);
        if output.strip() != "":
          try:
            int(output.strip(),16)
            if len(output) <= 2:
              output = bin(int(output.strip(),16))[2:].zfill(8)

            #print "this is the output: "+output
          except ValueError:
            output = output.strip()

          final_binary = final_binary + output+"\n"
    # print(final_binary)
    num_lines = len(final_binary.splitlines())
    # Add Segmentation and Partial Flash modes
    while num_lines<=255:
      final_binary = final_binary + "00000000"+"\n"
      num_lines+=1
    return final_binary,log_str
    # outFile = open("memory.bin","w")
    # outFile.write(final_binary)


"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
        # This is the name of your project. The first time you publish this
        # package, this name will be registered for you. It will determine how
        # users can install this project, e.g.:
        #
        # $ pip install sampleproject
        #
        # And where it will live on PyPI: https://pypi.org/project/sampleproject/
        #
        # There are some restrictions on what makes a valid project name
        # specification here:
        # https://packaging.python.org/specifications/core-metadata/#name
        name='RNBIP',  # Required

        # Versions should comply with PEP 440:
        # https://www.python.org/dev/peps/pep-0440/
        #
        # For a discussion on single-sourcing the version across setup.py and the
        # project code, see
        # https://packaging.python.org/en/latest/single_source_version.html
        version='1.6.1',  # Required

        # This is a one-line description or tagline of what your project does. This
        # corresponds to the "Summary" metadata field:
        # https://packaging.python.org/specifications/core-metadata/#summary
        description='RNBIP ISA Emulator',  # Optional

        # This is an optional longer description of your project that represents
        # the body of text which users will see when they visit PyPI.
        #
        # Often, this is the same as your README, so you can just read it in from
        # that file directly (as we have already done above)
        #
        # This field corresponds to the "Description" metadata field:
        # https://packaging.python.org/specifications/core-metadata/#description-optional
        long_description=long_description,  # Optional

        # Denotes that our long_description is in Markdown; valid values are
        # text/plain, text/x-rst, and text/markdown
        #
        # Optional if long_description is written in reStructuredText (rst) but
        # required for plain-text or Markdown; if unspecified, "applications should
        # attempt to render [the long_description] as text/x-rst; charset=UTF-8 and
        # fall back to text/plain if it is not valid rst" (see link below)
        #
        # This field corresponds to the "Description-Content-Type" metadata field:
        # https://packaging.python.org/specifications/core-metadata/#description-content-type-optional
        long_description_content_type='text/markdown',  # Optional (see note above)

        # This should be a valid link to your project's main homepage.
        #
    # This field corresponds to the "Home-Page" metadata field:
        # https://packaging.python.org/specifications/core-metadata/#home-page-optional
            url='https://github.com/digital-design-snu/RNBIP',  # Optional

    # This should be your name or the name of the organization which owns the
        # project.
            author='Paul George',  # Optional

    # This should be a valid email address corresponding to the author listed
        # above.
            author_email='command.paul@gmail.com',  # Optional

    # Classifiers help users find your project by categorizing it.
        #
            # For a list of valid classifiers, see https://pypi.org/classifiers/
                classifiers=[  # Optional
                        # How mature is this project? Common values are
                        #   3 - Alpha
                        #   4 - Beta
                        #   5 - Production/Stable
                        'Development Status :: 3 - Alpha',

                        # Indicate who your project is intended for
                        'Intended Audience :: Developers',

                        # Pick your license as you wish
                        'License :: OSI Approved :: MIT License',

                        # Specify the Python versions you support here. In particular, ensure
                        # that you indicate you support Python 3. These classifiers are *not*
                        # checked by 'pip install'. See instead 'python_requires' below.
                        'Programming Language :: Python :: 3',
                        'Programming Language :: Python :: 3.6',
                        'Programming Language :: Python :: 3.7',
                        'Programming Language :: Python :: 3.8',
                        'Programming Language :: Python :: 3.9',
                        'Programming Language :: Python :: 3 :: Only',
                    ],

    # This field adds keywords for your project which will appear on the
        # project page. What does your project relate to?
            #
                # Note that this is a list of additional keywords, separated
                    # by commas, to be used to assist searching for the distribution in a
                        # larger catalog.
                            keywords='sample, setuptools, development',  # Optional

    # When your source code is in a subdirectory under the project root, e.g.
        # `src/`, it is necessary to specify the `package_dir` argument.

    # You can just specify package directories manually here if your project is
        # simple. Or you can use find_packages().
            packages = find_packages(),
                # Alternatively, if you just want to distribute a single Python file, use
                    # the `py_modules` argument instead as follows, which will expect a file
                        # called `my_module.py` to exist:
                            #
                                # py_modules=["emulator_backend","emulator_gui","emulator_io","rnbip_MainWindow_UI",
                                #             "rnbip_assembler","rnbip_emulator","RNBIP_ISA"],

    # Specify which Python versions you support. In contrast to the
        # 'Programming Language' classifiers above, 'pip install' will check this
            # and refuse to install the project if the version does not match. See
                # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
                    python_requires='>=3.6, <4',

    # This field lists other packages that your project depends on to run.
        # Any package you put here will be installed by pip when your project is
            # installed, so they must be valid existing projects.
                #
                    # For an analysis of "install_requires" vs pip's requirements files see:
                        # https://packaging.python.org/en/latest/requirements.html
                            install_requires=['PyQt5'],  # Optional
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # package_data={'rnbip_emulator': ['asset/back_final.png','asset/Capture2.png','asset/Capture3.png','asset/Capture_final.png','asset/Capture_x.png','asset/C_no.png','asset/C_yes.png','asset/Databus.png','asset/D_SP.png','asset/E_FL.png','asset/E_IP.png','asset/E_OR.png','asset/E_PC.png','asset/E_R0.png','asset/E_RN.png','asset/E_SP.png','asset/I_PC.png','asset/I_SP.png','asset/killEOR.png','asset/killEPC.png','asset/killLPC.png','asset/killLR0.png','asset/killLRN2.png','asset/L_IR.png','asset/L_OP.png','asset/L_OR.png','asset/L_PC.png','asset/L_R0.png','asset/L_RN.png','asset/L_SP.png','asset/OD_PC.png','asset/OP_no.png','asset/OP_yes.png','asset/P_no.png','asset/P_yes.png','asset/R0_AS.png','asset/RD.png','asset/SAF_activated.png','asset/S_AL.png','asset/S_IF.png','asset/S_PC.png','asset/specialPath.png','asset/S_SP.png','asset/WR.png','asset/Z_no.png','asset/Z_yes.png','machCode.bin']},
    include_package_data=True,

    # To provide executable scripts, use entry points in preference to the
        # "scripts" keyword. Entry points provide cross-platform support and allow
            # `pip` to create the appropriate form of executable for the target
                # platform.
                    #
                        # For example, the following would provide a command called `sample` which
                            # executes the function `main` from this package when invoked:
                                entry_points={  # Optional
                                    'console_scripts': [
                                                    'rnbip=rnbip_emulator:main',
                                    ],
                                    },

)




import sys

from rnbip_emulator.rnbip_assembler import rnbip_assembler
from rnbip_emulator.emulator_backend import single_bus_emulator
from rnbip_emulator import emulator_gui

def main():
  inst_emulator  = single_bus_emulator()
  inst_assembler = rnbip_assembler()
  app = emulator_gui.QApplication(sys.argv)
  ex = emulator_gui.MainWidow(inst_emulator,inst_assembler)
  app.exec_()
  inst_emulator.end_emulation()
  sys.exit()



import random
import pdb
import os
import datetime
import platform
import re
import sys
import tkinter as tk

class IO_Interface(tk.Tk):
  def __init__(self,IO_In,IO_Out):
    tk.Tk.__init__(self)
    #self.C_IO_In = IO_In
    #self.C_IO_Out = IO_Out
    self.title('Virtual IO Interface')
    self.data = None
    self.entries = []
    for i in range(1,12):
      if i == 1:
        row = tk.Frame(self) 
        lab1 = tk.Label(row, width=8,text= "Port",anchor='center',borderwidth=2, relief="groove")
        lab2 = tk.Label(row, width=25,text="Output",anchor='center',borderwidth=2, relief="groove")
        lab3 = tk.Label(row, width=75,text="Input[current value]",anchor='center',borderwidth=2, relief="groove")
        row.pack( fill=tk.X, padx=5, pady=5)
        lab1.pack(side=tk.LEFT,fill=tk.X)
        lab2.pack(side=tk.LEFT,fill=tk.X,expand=tk.YES)
        lab3.pack(side=tk.LEFT,fill=tk.X,expand=tk.YES)
        #ent.pack(side=tk.RIGHT, expand=tk.YES, fill=tk.X)
        #self.entries.append(ent)
      elif(i==2):
        row = tk.Frame(self) 
        row.pack( fill=tk.X, padx=5, pady=5)
        lab1 = tk.Label(row, width=5,text="",anchor='center',justify = 'center',borderwidth=2, relief="flat")
        lab1.pack(side=tk.LEFT,expand=tk.YES)
        lab2 = tk.Label(row, width=5,text="HEX",anchor='center',justify = 'center',borderwidth=2, relief="raised")
        lab2.pack(side=tk.LEFT,expand=tk.YES)
        for j in range (7,-1,-1) :
          lab = tk.Label(row, width=1,text=str(j),anchor='center',justify = 'center')
          lab.pack(side=tk.LEFT,fill=tk.X,expand=tk.YES)
        lab = tk.Label(row, width=2,text="")
        lab.pack(side=tk.LEFT,expand=tk.YES)
        lab3 = tk.Label(row, width=6,text="HEX",anchor='center',justify = 'center',borderwidth=2, relief="raised")
        lab3.pack(side=tk.LEFT,expand=tk.YES)
        lab = tk.Label(row, width=1,text="")
        lab.pack(side=tk.LEFT,expand=tk.YES)
        for j in range (7,-1,-1) :
          lab = tk.Label(row, width=1,text="")
          lab.pack(side=tk.LEFT,expand=tk.YES)
          lab = tk.Label(row, width=3,text=str(j),anchor='center',justify = 'center')
          lab.pack(side=tk.LEFT,fill=tk.X,expand=tk.YES)
      elif(i==11):
        row = tk.Frame(self)
        row.pack( fill=tk.X, padx=5, pady=5,expand=tk.YES)
        self.update_Button = tk.Button(row, width=40, text='Commit Changes(if any) & Continue',command=lambda:self.update_io(IO_In,IO_Out)) 
        self.update_Button.pack(side=tk.BOTTOM,expand=tk.YES)
      else:
        row = tk.Frame(self) 
        row.pack( fill=tk.X, padx=5, pady=5,expand=tk.YES)
        lab1 = tk.Label(row, width=5,text=str(i-3),anchor='center',justify = 'center',borderwidth=2, relief="flat")
        lab1.pack(side=tk.LEFT,expand=tk.YES)
        lab2 = tk.Label(row, width=5,text=str(IO_Out[i-3]),anchor='center',justify = 'center',borderwidth=2, relief="raised")
        lab2.pack(side=tk.LEFT,expand=tk.YES)
        for j in range (7,-1,-1) :
          lab = tk.Label(row, width=1,text=str((IO_Out[i-3]&(1<<j))>>j),anchor='w')
          lab.pack(side=tk.LEFT,fill=tk.X,expand=tk.YES)
        lab = tk.Label(row, width=2,text="")
        lab.pack(side=tk.LEFT,expand=tk.YES)
        ent = tk.Entry(row,width = 7,justify = 'center',borderwidth=2, relief="raised")
        ent.insert(5,str(IO_In[i-3]))
        ent.pack(side=tk.LEFT, fill=tk.X,expand=tk.YES)
        ent8v = []
        for j in range (7,-1,-1) :
          lab = tk.Label(row, width=2,text="")
          lab.pack(side=tk.LEFT,expand=tk.YES)
          EntB = tk.Entry(row, width=3,justify = 'center')
          EntB.insert(3,str((IO_In[i-3]&(1<<j))>>j))
          EntB.pack(side=tk.LEFT,fill=tk.X,expand=tk.YES)				
          ent8v.append(EntB)
        self.entries.append((ent,ent8v))
    #self.update_Button.pack(row,expand=tk.YES, fill=tk.X)
    #self.update_Button.grid(row=1, column=1, padx=8, pady=8)
  
  def update_io(self,IO_In,IO_Out):
    for i in range(0,8):
      k = 0
      for j in range (0,8):	
        # Reconstruct the Byte Here
        inp = int(self.entries[i][1][j].get())
        if(inp != 0 )and(inp != 1):
          return -1
        k = inp +2*k;
      hexk = int(self.entries[i][0].get())
      if(hexk!=k and hexk == IO_In[i]):
        IO_In[i]=k
      elif(hexk!=k and k == IO_In[i]):
        IO_In[i] = hexk
      #IO_In[i] = self.entries[i]
    self.destroy()
